#!/bin/bash

# Counts ownCloud licenses currently in use.
# Arguments:
#
#	(nothing)	print results to STDOUT
#	-m			mail to predefined recipients
#	-m <addr>	mail to this address 

usage()
{
	echo "Usage: `basename $0` [ -m [ <addr> ] ]"
	exit 1
}

TO=`ucr get owncloud/report/to`
[ -z "${TO}" ] && TO='license-report@it25.de'
CC=`ucr get mail/alias/root`
ME="License Report <${CC}>"

if [ -n "$1" ] ; then
	if [ "$1" != '-m' ] ; then
		echo "ERROR: Illegal argument '$1'" >&2
		usage
	fi
	shift
	
	if [ -n "$1" ] ; then
		if ! expr "$1" : '.*@' &>/dev/null; then
			echo "ERROR: Not a mail address: '$1'" >&2
			usage
		fi
		TO="$1"
		shift
	fi
	
	if [ -n "$1" ] ; then
		echo "ERROR: Too much arguments" >&2
		usage
	fi
	
	MSG="Message sent to ${TO}"
	if [ -n "${CC}" ] ; then
		MSG="${MSG} and ${CC}"
		CC="-c ${CC}"
	fi
	
	SUB="ownCloud license usage [`date -R`]"

	$0 | mail -r "${ME}" -s "${SUB}" ${CC} ${TO}
	
	[ -t 1 ] && echo ${MSG}
	
	exit
fi

echo "ownCloud license usage on `date -R`"
echo

L_HN=`hostname --fqdn`
L_LI=`php -r "include('/var/www/owncloud/config/config.php'); print \\\$CONFIG['license-key'];"`
L_US=`su -c '/usr/bin/php /var/www/owncloud/occ user:report' www-data | grep 'total users' | grep -oE '[0-9]+'`

echo "  host name ....... ${L_HN}"
echo "  license ......... ${L_LI}"
echo "  users in use .... ${L_US}"

