<?php
/**
 * ownCloud
 *
 * @author Tom Needham <tom@owncloud.com>
 * @author Lukas Reschke <lukas@owncloud.com>
 * @copyright (C) 2015 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

namespace OCA\Files_Drop\Controller;

use OCA\Files_Drop\Lib\HelperWrapper;
use OCP\AppFramework\Controller;
use OCP\AppFramework\Http;
use OCP\Files\IRootFolder;
use OCP\IConfig;
use OCP\IRequest;
use OCP\AppFramework\Http\DataResponse;
use OCA\Files_Drop\Lib\EndpointService;
use OCP\Files\Folder;

/**
 * Class EndpointController
 *
 * @package OCA\Files_Drop\Controller
 */
Class EndpointController extends Controller {
	/** @var string */
	private $userId;
	/** @var EndpointService */
	private $endpointService;
	/** @var IConfig */
	private $config;
	/** @var IRootFolder */
	private $rootFolder;
	/** @var HelperWrapper */
	private $helperWrapper;

	/**
	 * @param string $AppName
	 * @param IRequest $request
	 * @param string $UserId
	 * @param EndpointService $endpointService
	 * @param IConfig $config
	 * @param IRootFolder $rootFolder
	 * @param HelperWrapper $helperWrapper
	 */
	public function __construct($AppName,
								$UserId,
								IRequest $request,
								EndpointService $endpointService,
								IConfig $config,
								IRootFolder $rootFolder,
								HelperWrapper $helperWrapper) {
		parent::__construct($AppName, $request);
		$this->userId = $UserId;
		$this->endpointService = $endpointService;
		$this->config = $config;
		$this->rootFolder = $rootFolder;
		$this->helperWrapper = $helperWrapper;
	}

	/**
	 * @param string $path The path to save
	 * @return DataResponse
	 *
	 * @NoAdminRequired
	 */
	public function create($path) {
		try {
			$endpoint = $this->endpointService->getByUser($this->userId);
		} catch (\Exception $exception) {
			return new DataResponse(['message' => 'Error occurred'], Http::STATUS_INTERNAL_SERVER_ERROR);
		}

		/** @var Folder $folderView */
		$folderView = $this->rootFolder->get($this->userId .'/files/');

		try {
			$fileId = $folderView->get($path)->getId();
			$endpoint->setFileId($fileId);
			$endpoint->setToken($this->endpointService->getUnusedToken());
			$this->endpointService->save($endpoint);
		} catch (\Exception $e) {
			return new DataResponse(['message' => 'Couldn\'t find file ID']);
		}

		// Return the url to the UI
		return new DataResponse([
			'url' => $endpoint->getUrl(),
			'path' => $endpoint->getRelativePath(),
			'token' => $endpoint->getToken(),
			'freeSpace' => $this->helperWrapper->freeSpace($endpoint->getRelativePath()),
			'maxUpload' => $this->helperWrapper->maxUploadFilesize($endpoint->getRelativePath()),
			]
		);
	}

	/**
	 * @return DataResponse
	 *
	 * @NoAdminRequired
	 */
	public function delete() {
		try {
			$endpoint = $this->endpointService->getByUser($this->userId);
			$this->endpointService->delete($endpoint);
		} catch (\Exception $e) {
			return new DataResponse(['message' => 'EndpointService not found'], Http::STATUS_NOT_FOUND);
		}

		return new DataResponse();
	}

}
