/**
 * ownCloud
 *
 * @author Tom Needham <tom@owncloud.com>
 * @author Lukas Reschke <lukas@owncloud.com>
 * @copyright (C) 2015 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

(function ($, OC) {

    if (!OCA.Files_Drop) {
        OCA.Files_Drop = {};
    }

    var App = {

        initialize: function () {
            // Choose button
            $('form.files_drop button.choose').click(function (e) {
                $('form.files_drop p.error').text('');
                e.preventDefault();
                OC.dialogs.filepicker(
                    t('files_drop', 'Choose the folder to store files uploaded with File Drop'),
                    OCA.Files_Drop.savePath,
                    false,
                    'httpd/unix-directory'
                );
            });

            // Remove endpoint button
            $('form.files_drop button.remove').click(function (e) {
                $('form.files_drop p.error').text('');
                e.preventDefault();
                OCA.Files_Drop.removeEndpoint($('form.files_drop input.url').attr('data-token'));
            });

            // Auto select text in url box
            $('form.files_drop input.url').click(function () {
                $(this).select();
            });
        },

        /**
         * Saves the path for file drop storage to the database via ajax
         * @param {string} path
         */
        savePath: function (path) {
            // Check if file or directory selected.
            if (path.substr(-1) !== '/') {
                // Must have picked a file, not a folder!
                $('form.files_drop span.access_url').text(t('files_drop', 'Error: Please select a folder, not a file.'));
                $('form.files_drop span.access_url').addClass('error');
                return;
            }
            $.ajax(OC.generateUrl('/apps/files_drop/endpoint'), {
                data: {'path': path},
                type: 'POST',
                success: function (jsondata) {
                    $('form.files_drop div.endpoint_new').hide();
                    $('form.files_drop div.endpoint_existing input.url').val(escapeHTML(jsondata.url))
                        .attr('data-token', escapeHTML(jsondata.token));
                    $('form.files_drop div.endpoint_existing span.endpoint_path').text(jsondata.path);
                    $('form.files_drop div.endpoint_existing span.max_upload').text(humanFileSize(jsondata.maxUpload));
                    $('form.files_drop div.endpoint_existing span.free_space').text(humanFileSize(jsondata.freeSpace));
                    $('form.files_drop div.endpoint_existing').show();
                },
                error: function () {
                    $('form.files_drop p.error').show().text(t('files_drop', 'An error occurred'));
                }
            });
        },

        /**
         *  Remove the files drop endpoint
         *
         *  @param {string} token
         */
        removeEndpoint: function (token) {
            $.ajax(OC.generateUrl('/apps/files_drop/endpoint'), {
                data: {'token': token},
                type: 'DELETE',
                success: function () {
                    $('form.files_drop div.endpoint_existing').hide();
                    $('form.files_drop button.remove').hide();
                    $('form.files_drop div.endpoint_new').show();
                },
                error: function () {
                    $('form.files_drop p.error').text(t('files_drop', 'An error occurred'));
                }
            });
        }

    };
    OCA.Files_Drop = App;

    // Go go go
    $(document).ready(function() {
        OCA.Files_Drop.initialize();
    });

})(jQuery, OC);