<?php
/**
 * ownCloud
 *
 * @author Lukas Reschke <lukas@owncloud.com>
 * @copyright (C) 2015 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

namespace OCA\Files_Drop\tests\unit\lib;

use OCA\Files_Drop\Lib\Endpoint;
use Test\TestCase;
use OCA\Files_Drop\Lib\EndpointService;
use OCP\IConfig;
use OCP\IURLGenerator;
use OCP\Files\IRootFolder;
use OCP\Security\ISecureRandom;
use OCA\Files_Drop\Lib\HelperWrapper;

/**
 * Class EndpointServiceTest
 *
 * @package OCA\Files_Drop\tests\unit\lib
 */
class EndpointServiceTest extends TestCase {
	/** @var EndpointService */
	private $endpointService;
	/** @var string */
	private $appName;
	/** @var IConfig */
	private $config;
	/** @var IURLGenerator */
	private $urlGenerator;
	/** @var IRootFolder */
	private $rootFolder;
	/** @var ISecureRandom */
	private $secureRandom;
	/** @var HelperWrapper */
	private $helperWrapper;

	public function setUp() {
		parent::setUp();

		$this->appName = 'files_drop';
		$this->config = $this->getMockBuilder('\OCP\IConfig')
			->disableOriginalConstructor()->getMock();
		$this->urlGenerator = $this->getMockBuilder('\OCP\IURLGenerator')
			->disableOriginalConstructor()->getMock();
		$this->rootFolder = $this->getMockBuilder('\OCP\Files\IRootFolder')
			->disableOriginalConstructor()->getMock();
		$this->secureRandom = $this->getMockBuilder('\OCP\Security\ISecureRandom')
			->disableOriginalConstructor()->getMock();
		$this->helperWrapper = $this->getMockBuilder('\OCA\Files_Drop\Lib\HelperWrapper')
			->disableOriginalConstructor()->getMock();

		$this->endpointService = new EndpointService(
			$this->appName,
			$this->config,
			$this->urlGenerator,
			$this->rootFolder,
			$this->secureRandom,
			$this->helperWrapper
		);
	}

	public function testGetByUser() {
		$userFolder = $this->getMockBuilder('\OCP\Files\Folder')
			->disableOriginalConstructor()->getMock();
		$this->rootFolder
			->expects($this->once())
			->method('get')
			->with('björn.schießle/files/')
			->will($this->returnValue($userFolder));

		$expected = new Endpoint(
			'björn.schießle',
			'files_drop',
			$this->config,
			$this->urlGenerator,
			$userFolder,
			$this->helperWrapper
		);

		$this->assertEquals($expected, $this->endpointService->getByUser('björn.schießle'));
	}

	/**
	 * @expectedException \OCP\Files\NotFoundException
	 */
	public function testGetByTokenException() {
		$userFolder = $this->getMockBuilder('\OCP\Files\Folder')
			->disableOriginalConstructor()->getMock();
		$this->config
			->expects($this->once())
			->method('getUsersForUserValue')
			->with($this->appName, 'token', 'TheToken')
			->will($this->returnValue([]));

		$expected = new Endpoint(
			'björn.schießle',
			'files_drop',
			$this->config,
			$this->urlGenerator,
			$userFolder,
			$this->helperWrapper
		);

		$this->assertEquals($expected, $this->endpointService->getByToken('TheToken'));
	}

	public function testGetByToken() {
		$userFolder = $this->getMockBuilder('\OCP\Files\Folder')
			->disableOriginalConstructor()->getMock();
		$this->config
			->expects($this->once())
			->method('getUsersForUserValue')
			->with($this->appName, 'token', 'TheToken')
			->will($this->returnValue(['björn.schießle']));
		$this->rootFolder
			->expects($this->once())
			->method('get')
			->with('björn.schießle/files/')
			->will($this->returnValue($userFolder));

		$expected = new Endpoint(
			'björn.schießle',
			'files_drop',
			$this->config,
			$this->urlGenerator,
			$userFolder,
			$this->helperWrapper
		);

		$this->assertEquals($expected, $this->endpointService->getByToken('TheToken'));
	}

	public function testSave() {
		$endpoint = $this->getMockBuilder('\OCA\Files_Drop\Lib\Endpoint')
			->disableOriginalConstructor()->getMock();
		$endpoint
			->expects($this->exactly(2))
			->method('getUser')
			->will($this->returnValue('björn.schießle'));
		$endpoint
			->expects($this->once())
			->method('getFileId')
			->will($this->returnValue(1337));
		$endpoint
			->expects($this->once())
			->method('getToken')
			->will($this->returnValue('TheToken'));
		$this->config
			->expects($this->at(0))
			->method('setUserValue')
			->with('björn.schießle', $this->appName, 'fileId', 1337);
		$this->config
			->expects($this->at(1))
			->method('setUserValue')
			->with('björn.schießle', $this->appName, 'token', 'TheToken');

		$this->endpointService->save($endpoint);
	}

	public function testDelete() {
		$endpoint = $this->getMockBuilder('\OCA\Files_Drop\Lib\Endpoint')
			->disableOriginalConstructor()->getMock();
		$endpoint
			->expects($this->exactly(2))
			->method('getUser')
			->will($this->returnValue('björn.schießle'));
		$this->config
			->expects($this->at(0))
			->method('deleteUserValue')
			->with('björn.schießle', $this->appName, 'fileId');
		$this->config
			->expects($this->at(1))
			->method('deleteUserValue')
			->with('björn.schießle', $this->appName, 'token');

		$this->endpointService->delete($endpoint);
	}
}
