<?php
/**
 * ownCloud
 *
 * @author Frank Karlitscheck <frank@owncloud.com>
 * @author Jörn Friedrich Dreyer <jfd@owncloud.com>
 * @author Tom Needham <tom@owncloud.com>
 * @author Michael Gapczynski
 * @copyright (C) 2014 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

namespace OCA\Files_Sharing_Log;

use \OCP\Share;
use \OCP\User;
use \OCP\Util;

class Log {
	static public function fopen($arguments) {
		$target = $arguments['target'];
		$source = $arguments['source'];
		$mode = $arguments['mode'];
		self::log($target, $source, $mode);
	}

	static public function file_get_contents($arguments) {
		$target = $arguments['target'];
		$source = $arguments['source'];
		$mode = 'get';
		self::log($target, $source, $mode);
	}

	static public function file_put_contents($arguments) {
		$target = $arguments['target'];
		$source = $arguments['source'];
		$mode = 'put';
		self::log($target, $source, $mode);
	}

	static public function post_shared($arguments) {
		$itemType = $arguments['itemType'];
		if ($itemType === 'file' || $itemType === 'folder') {
			$uidOwner = $arguments['uidOwner'];
			$shareWith = $arguments['shareWith'];
			$shareType = $arguments['shareType'];
			$file = $arguments['fileTarget'];
			$permissions = $arguments['permissions'];
			$msg = 'User '.$uidOwner.' shared "'.$file.'" with ';
			if ($shareType === Share::SHARE_TYPE_USER) {
				$msg .= 'the user '.$shareWith;
			} else if ($shareType === Share::SHARE_TYPE_GROUP) {
				$msg .= 'the group '.$shareWith;
			} else if ($shareType === Share::SHARE_TYPE_LINK) {
				$msg .= 'a public link, token='.$arguments['token'];;
			}
			$msg .= ', permissions:';
			if ($permissions & \OCP\PERMISSION_READ) {
				$msg .= ' READ';
			}
			if ($permissions & \OCP\PERMISSION_UPDATE) {
				$msg .= ' UPDATE';
			}
			if ($permissions & \OCP\PERMISSION_CREATE) {
				$msg .= ' CREATE';
			}
			if ($permissions & \OCP\PERMISSION_DELETE) {
				$msg .= ' DELETE';
			}
			if ($permissions & \OCP\PERMISSION_SHARE) {
				$msg .= ' SHARE';
			}
			Util::writeLog('sharing_log', $msg, Util::INFO);
		}
	}

	static public function post_email($arguments) {
		$uidOwner = $arguments['uidOwner'];
		$file = $arguments['file'];
		$link = $arguments['link'];
		$toAddress = $arguments['toAddress'];
		Util::writeLog('sharing_log', 'User '.$uidOwner.' sent '.$file.' via '.$link
			.' to recipient '.$toAddress, Util::INFO
		);
	}

	static public function post_update_permissions($arguments) {
		$itemType = $arguments['itemType'];
		if ($itemType === 'file' || $itemType === 'folder') {
			$uidOwner = $arguments['uidOwner'];
			$shareWith = $arguments['shareWith'];
			$shareType = $arguments['shareType'];
			$file = $arguments['path'];
			$permissions = $arguments['permissions'];
			$msg = 'User '.$uidOwner.' updated the permissions for "'.$file.'" for ';
			if ($shareType === Share::SHARE_TYPE_USER) {
				$msg .= 'the user '.$shareWith;
			} else if ($shareType === Share::SHARE_TYPE_GROUP) {
				$msg .= 'the group '.$shareWith;
			} else if ($shareType === Share::SHARE_TYPE_LINK) {
				$msg .= 'a public link, token='.$arguments['token'];;
			}
			$msg .= ', permissions:';
			if ($permissions & \OCP\PERMISSION_READ) {
				$msg .= ' READ';
			}
			if ($permissions & \OCP\PERMISSION_UPDATE) {
				$msg .= ' UPDATE';
			}
			if ($permissions & \OCP\PERMISSION_CREATE) {
				$msg .= ' CREATE';
			}
			if ($permissions & \OCP\PERMISSION_DELETE) {
				$msg .= ' DELETE';
			}
			if ($permissions & \OCP\PERMISSION_SHARE) {
				$msg .= ' SHARE';
			}
			Util::writeLog('sharing_log', $msg, Util::INFO);
		}
	}

	static public function post_unshare($arguments) {
		$shareType = $arguments['shareType'];
		$uidOwner = $arguments['uidOwner'];
		$itemType = $arguments['itemType'];
		$file = $arguments['fileTarget'];
		$shareWith = $arguments['shareWith'];

		if($shareType == Share::SHARE_TYPE_USER || $shareType == Share::SHARE_TYPE_GROUP) {
			$targetMsg = $shareType == Share::SHARE_TYPE_USER ? 'user \'' : 'group \'';
			$msg = 'User \'' . $uidOwner . '\' unshared their ' . $itemType . ' (' . $file . ') with '
				. $targetMsg . $shareWith . '\'';
		} else if($shareType == Share::SHARE_TYPE_LINK) {
			$msg = 'User \'' . $uidOwner . '\' removed the share link for their ' . $itemType . ' (' . $file . ')';
		} else {
			$msg = 'User \'' . $uidOwner . '\' unshared their ' . $itemType . '(' . $file . ') with \''
				. $shareWith . '\'';
		}

		Util::writeLog('sharing_log', $msg, Util::INFO);
	}


	static public function log($target, $source, $mode) {
		$msg = "User " . User::getUser() . " performed action \"$mode\" on file $source";
		Util::writeLog('sharing_log', $msg, Util::INFO);
	}
}
