<?php
/**
 * ownCloud
 *
 * @author Juan Pablo Villafañez Ramos <jvillafanez@owncloud.com>
 * @author Jesus Macias Portela <jesus@owncloud.com>
 * @copyright (C) 2014 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */
use \OCA\windows_network_drive\lib\DBAccess;
use \OCA\windows_network_drive\lib\Utils;

\OCP\JSON::checkAppEnabled('windows_network_drive');
\OCP\User::checkAdminUser();
\OCP\JSON::callCheck();

$validkeys = array('personal_mounts', 'global_sharing');

$l = OC_L10N::get('windows_network_drive');

if (!isset($_POST['key']) || !in_array($_POST['key'], $validkeys)) {
    \OCP\JSON::error(array("data" => array("message" => $l->t('Invalid key'))));
    exit();
}

$key = $_POST['key'];
$active = (isset($_POST['activate']) && $_POST['activate'] === 'true') ? true : false;

\OCP\Config::setAppValue('windows_network_drive', $key, $active);

if ($key === 'global_sharing') {
    $cursor = DBAccess::getAllMountsWithCredentials();
    if ($cursor !== false) {
        while (($row = $cursor->fetchRow()) !== false) {
            $wnd = \OCA\windows_network_drive\lib\WNDUtils::getWndForGlobalRow($row);
            if ($wnd) {
                $wnd->getCache()->clear();
            }
        }
    } else {
        \OCP\JSON::error(array("data" => array("message" => $l->t('Cannot connect to the DB'))));
        exit();
    }

    $cursor = DBAccess::getAllPersonalMounts();
    if ($cursor !== false) {
        while (($row = $cursor->fetchRow()) !== false) {
            $wnd = \OCA\windows_network_drive\lib\WNDUtils::getWndForPersonalRow($row);
            if ($wnd) {
                $wnd->getCache()->clear();
            }
        }
    } else {
        \OCP\JSON::error(array("data" => array("message" => $l->t('Cannot connect to the DB'))));
        exit();
    }

}

if ($key === 'personal_mounts' && !$active) {
    $cursor = DBAccess::getAllPersonalMounts();
    if ($cursor !== false) {
        // clear the cache and storage data before removing the information from the DB
        while (($row = $cursor->fetchRow()) !== false) {
            $wnd = \OCA\windows_network_drive\lib\WNDUtils::getWndForPersonalRow($row);
            if ($wnd) {
                $wnd->getCache()->clear();
            }
        }
        DBAccess::clearPersonalMounts();
    } else {
        \OCP\JSON::error(array("data" => array("message" => $l->t('Cannot connect to the DB'))));
        exit();
    }
}
\OCP\JSON::success(array("data" => array("message" => $l->t('ok'))));
