<?php
/**
 * ownCloud
 *
 * @author Juan Pablo Villafañez Ramos <jvillafanez@owncloud.com>
 * @author Jesus Macias Portela <jesus@owncloud.com>
 * @copyright (C) 2014 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */
use \OCA\windows_network_drive\lib\DBAccess;
use \OCA\windows_network_drive\lib\Utils;

\OCP\JSON::checkAppEnabled('windows_network_drive');
if ($_POST['type'] === 'admin') {
    \OCP\User::checkAdminUser();
} else {
    \OCP\User::checkLoggedIn();
}
\OCP\JSON::callCheck();

$postUser = Utils::getDefault($_POST, 'WNDuser', '');
$postPass = Utils::getDefault($_POST, 'WNDpass', '');

$l = OC_L10N::get('windows_network_drive');
$user = \OCP\User::getUser();
$password = Utils::encryptPassword($postPass);
if ($_POST['type'] === 'admin') {
    $adminCursor = DBAccess::getAdminGlobalCredentials();
    if ($adminCursor && ($adminCursor->fetchRow()) !== false) {
        if (isset($_POST['WNDuser'])) {
            DBAccess::updateGlobalCredentials($user, 'global_username', $postUser, false);
        }
        if (isset($_POST['WNDpass'])) {
            DBAccess::updateGlobalCredentials($user, 'global_password', $password, false);
        }
    } else {
        DBAccess::saveAdminGlobalCredentials($user, $postUser, $password);
    }
} else {
    if (isset($_POST['WNDMountId'])) {
        // if we have a mount id, check if we can access to that mount with those
        // credentials before saving them in the DB
        $cursor = DBAccess::getPersonalMountByIdAndUser(intval($_POST['WNDMountId']), $user);
        if ($cursor !== false) {
            if (($row = $cursor->fetchRow()) !== false) {
                $params = array('host' => $row['mount_url'],
                                'share' => $row['share'],
                                'root' => $row['root'],
                                'user' => $postUser,
                                'password' => $postPass);
                $wnd = new \OCA\windows_network_drive\lib\WND($params);
                try {
                    if ($wnd->test()) {
                        DBAccess::savePersonalGlobalCredentials($user, $postUser, $password);
                    } else {
                        \OCP\JSON::error(array("data" => array("message" => $l->t('Cannot access to the server with the provided data'))));
                        exit();
                    }
                } catch (\Exception $e) {
                    \OCP\Util::logException('wnd', $e);
                    \OCP\JSON::error(array("data" => array("message" => $l->t('Cannot access to the server with the provided data'))));
                    exit();
                }
            }
        }
    } else {
        // save without checking
        $personalCursor = DBAccess::getPersonalGlobalCredentials($user);
        if ($personalCursor && ($personalCursor->fetchRow()) !== false) {
            if (isset($_POST['WNDuser'])) {
                DBAccess::updateGlobalCredentials($user, 'global_username', $postUser, true);
            }
            if (isset($_POST['WNDpass'])) {
                DBAccess::updateGlobalCredentials($user, 'global_password', $password, true);
            }
        } else {
            DBAccess::savePersonalGlobalCredentials($user, $postUser, $password);
        }
    }
}

\OCP\JSON::success(array("data" => $l->t('ok')));
