<?php
/**
 * ownCloud
 *
 * @author Juan Pablo Villafañez Ramos <jvillafanez@owncloud.com>
 * @author Jesus Macias Portela <jesus@owncloud.com>
 * @copyright (C) 2014 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */
use \OCA\windows_network_drive\lib\DBAccess;
use \OCA\windows_network_drive\lib\Utils;

\OCP\JSON::checkAppEnabled('windows_network_drive');
\OCP\User::checkAdminUser();
\OCP\JSON::callCheck();

$l = OC_L10N::get('windows_network_drive');
$mandatoryParams = array('WNDMountId', 'type');
foreach ($mandatoryParams as $mparam) {
    if (!isset($_POST[$mparam]) || ($_POST[$mparam] !== "0" && empty($_POST[$mparam]))) {
        \OCP\JSON::error(array("data" => array("message" => $l->t('Missing mandatory parameters'))));
        exit();
    }
}

$_POST['WNDGuser'] = Utils::getDefault($_POST, 'WNDGuser', '');
$_POST['WNDGpass'] = Utils::getDefault($_POST, 'WNDGpass', '');

$user = \OCP\User::getUser();
// if type = 0 we'll use the credentials passed as parameter, type = 2 we'll use the global
// credentials for the admin
if (intval($_POST['type']) === 0) {
    $cuser = $_POST['WNDGuser'];
    $cpass = $_POST['WNDGpass'];
} else if (intval($_POST['type']) === 1) {
    // depends on the user personal credentials, so we'll just save and exit
    DBAccess::updateAdminMountCredentialsType(intval($_POST['WNDMountId']), intval($_POST['type']), null, null);
    \OCP\JSON::success(array("data" => $l->t('ok')));
    exit();
} else if (intval($_POST['type']) === 2) {
    $globalCredentials = DBAccess::getAdminGlobalCredentialsCached();
    $cuser = $globalCredentials['global_username'];
    $cpass = Utils::decryptPassword($globalCredentials['global_password']);
    if ($cpass === false) {
        \OC_Log::write('wnd', 'cannot decrypt: ' . $globalCredentials['global_password'], \OC_Log::ERROR);
    }
} else if (intval($_POST['type']) === 3) {
    DBAccess::updateAdminMountCredentialsType(intval($_POST['WNDMountId']), intval($_POST['type']), null, null);
    \OCP\JSON::success(array("data" => $l->t('ok')));
    exit();
} else {
    \OCP\JSON::error(array("data" => array("message" => $l->t('Unknown credential type'))));
    exit();
}

$cursor = DBAccess::getMountById(intval($_POST['WNDMountId']));
if ($cursor !== false) {
    // we'll only access to the first row (mount id should be unique anyway)
    if (($row = $cursor->fetchRow()) !== false) {
        $params = array('host' => $row['mount_url'],
                        'share' => $row['share'],
                        'root' => $row['root'],
                        'user' => $cuser,
                        'password' => $cpass);
        $wnd = new \OCA\windows_network_drive\lib\WND($params);
        try {
            if ($wnd->test()) {
                // clear cache data of old wnd mount point (we could have duplicated info in the DB)
                // for the case "let users use their credentials" we should also clean all those
                // mounts because they could be using different OC storage ids
                $cursor2 = DBAccess::getMountByIdWithCreds(intval($_POST['WNDMountId']));
                if ($cursor2 !== false) {
                    while (($row2 = $cursor2->fetchRow()) !== false) {
                        $oldwnd = \OCA\windows_network_drive\lib\WNDUtils::getWndForGlobalRow($row2);
                        if ($oldwnd) {
                            $oldwnd->getCache()->clear();
                        }
                    }
                }

                $encryptedPass = Utils::encryptPassword($cpass);
                DBAccess::updateAdminMountCredentialsType(intval($_POST['WNDMountId']), intval($_POST['type']), $cuser, $encryptedPass);
            } else {
                \OCP\JSON::error(array("data" => array("message" => $l->t('Cannot access to the server with the provided data'))));
                exit();
            }
        } catch (\Exception $e) {
            \OCP\Util::logException('wnd', $e);
            \OCP\JSON::error(array("data" => array("message" => $l->t('Cannot access to the server with the provided data'))));
            exit();
        }
    } else {
        \OCP\JSON::error(array("data" => array("message" => $l->t('Mount point not found'))));
        exit();
    }
} else {
    \OCP\JSON::error(array("data" => array("message" => $l->t('Error accessing data'))));
    exit();
}

\OCP\JSON::success(array("data" => $l->t('ok')));
