/**
 * ownCloud
 *
 * @author Juan Pablo Villafañez Ramos <jvillafanez@owncloud.com>
 * @author Jesus Macias Portela <jesus@owncloud.com>
 * @copyright (C) 2014 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

function TableCodeGenerator () {
    this.baseTemplate = null;
    this.generateTableRow2 = function(mount, callback, errorCallback) {
        var _self = this;
        if (typeof callback !== 'function') {
            return;
        }

        if (this.baseTemplate) {
            callback($(this.baseTemplate).octemplate(mount));
        } else if (_self.retrievingTemplate) {
            _self._generateTableRow2Request.always(function(){
                callback($(_self.baseTemplate).octemplate(mount));
            });
        } else {
            _self.retrievingTemplate = true;
            _self._generateTableRow2Request = $.ajax({
                type : 'GET',
                url : OC.filePath('windows_network_drive', '', 'jsTemplate.php'),
                data : {template : 'settingTableRow'},
                success : function(response) {
                    _self.baseTemplate = response;
                    callback($(_self.baseTemplate).octemplate(mount));
                },
                error : errorCallback,
                complete : function() {
                    _self.retrievingTemplate = false;
                }
            });
        }
    };

    this.editRow = function(trrow, active) {
        trrow.find('.td_wnd_creds').find('select, input').prop('disabled', !active);
        trrow.find('.td_wnd_creds select.wnd-select').change();
    };
}

function TableManager(tbody) {
    this.tbody = tbody;
    this.wndAjaxCalls = new WndAjaxCalls();
    this.codeGenerator = new TableCodeGenerator();

    this.attachDeleteHandlers = function(baseElement) {
        var wnd = this.wndAjaxCalls;
        if (baseElement === undefined) {
            baseElement = this.tbody;
        }
        baseElement.find('input[type="button"][name="delete"]').click(function(){
            var trrow = $(this).parent().parent();
            wnd.deleteMount(trrow.data('mid'),
                function(response){
                    if (response.status === 'success') {
                        trrow.remove();
                    }
                }, function(jqhrx, errorStatus, errorMessage){
                    OC.dialogs.alert(escapeHTML(jqhrx.status + ": " + errorMessage),
                              escapeHTML(t('windows_network_drive', 'Internal error')));

                });
        });
    };

    this.attachEditHandlers = function(baseElement) {
        var codeGen = this.codeGenerator;
        var wnd = this.wndAjaxCalls;
        var currentTableManager = this;
        if (baseElement === undefined) {
            baseElement = this.tbody;
        }
        baseElement.find('input[type="button"][name="edit"]').on('click.edit', function editClick(){
            var trrow = $(this).parent().parent();
            var tdCreds = trrow.find('.td_wnd_creds');
            tdCreds.data('oldCredInfo', tdCreds.html());
            tdCreds.find('select.wnd-select').change(function(){
                // the handler will disapear if clicked outside or updated
                var thisElement = $(this);
                if (thisElement.val() === "0") {
                    thisElement.siblings('input[name="WNDGuser"],input[name="WNDGpass"]').show();
                } else {
                    thisElement.siblings('input[name="WNDGuser"],input[name="WNDGpass"]').hide().val('');
                }

            });
            var eventId = Math.random().toString().slice(2);
            $(document).on('click.wndoutside.' + eventId, function(e){
                if (!$(e.target).closest(trrow).length) {
                    // reset the chosen because js handlers won't be in place
                    tdCreds.find('select.wnd-select').select2('destroy');
                    tdCreds.html(tdCreds.data('oldCredInfo'));

                    tdCreds.find('div.wnd-select').remove();
                    tdCreds.find('select.wnd-select').select2({minimumResultsForSearch: -1, width: '160px'});

                    trrow.find('input[type="button"][name="edit"]').val(t('windows_network_drive', 'Edit'))
                                                                    .off('click.update')
                                                                    .on('click.edit', editClick);
                    $(document).off('click.wndoutside.' + eventId);
                }
            });
            codeGen.editRow(trrow, true);
            $(this).off('click.edit')
                    .val(t('windows_network_drive', 'Update'))
                    .on('click.update', function(){
                            var user = trrow.find('input[name="WNDGuser"]').val();
                            var pwd = trrow.find('input[name="WNDGpass"]').val();
                            var type = trrow.find('td.td_wnd_creds select').val();
                            wnd.updateMountWithType(trrow.data('mid'), type, user, pwd, function(response){
                                    if (response.status === "success") {
                                        //currentTableManager.reloadTable();
                                        codeGen.editRow(trrow, false);
                                        trrow.find('input[type="button"][name="edit"]')
                                                                    .val(t('windows_network_drive', 'Edit'))
                                                                    .off('click.update')
                                                                    .on('click.edit', editClick);
                                        $(document).off('click.wndoutside.' + eventId);
                                    } else {
                                        if (response.data && response.data.message) {
                                            OC.dialogs.alert(response.data.message,
                                                        escapeHTML(t('windows_network_drive', 'Error checking Windows network drive configuration')));
                                        } else {
                                            OC.dialogs.alert(escapeHTML(t('windows_network_drive', 'Internal server error')),
                                                        escapeHTML(t('windows_network_drive', 'Error checking Windows network drive configuration')));
                                        }
                                    }
                            }, function(jqhrx, errorStatus, errorMessage){
                                OC.dialogs.alert(escapeHTML(jqhrx.status + ": " + errorMessage),
                                          escapeHTML(t('windows_network_drive', 'Internal error')));

                            });
            });
        });
    };

    this._postAddMountFix = function(template, mount) {
        if (mount.global_username && parseInt(mount.credType) === 0) {
            template.find('input[name="WNDGuser"]').val(mount.global_username);
        } else {
            template.find('input[name="WNDGuser"],input[name="WNDGpass"]').css('display', 'none');
        }
        template.find("select.wnd-select option[value=\"" + mount.credType + "\"]")
                .attr("selected", "selected")
                .prop("selected", true);
    }

    this.reloadTable = function() {
        var codeGen = this.codeGenerator;
        var targetBody = this.tbody;
        var currentTableManager = this;
        this.wndAjaxCalls.getAllMounts(function(response){
            if (response.status === 'success') {
                var allRows = [];
                var deferredList = [];
                $.each(response.data, function(index, mount){
                    var deferred = $.Deferred();
                    deferredList.push(deferred);
                    mount.credType = mount.use_custom_credentials;
                    mount.applicable_values = $.map(mount.applicables, function(obj){
                            if (obj === 'global') {
                                return 'All users';
                            } else {
                                return obj;
                            }
                    }).join();

                    codeGen.generateTableRow2(mount, function(m){
                        _.defer(function(){
                            allRows.push(m);
                            currentTableManager._postAddMountFix(m, mount);
                            deferred.resolve();
                        });
                    });
                });
                $.when.apply($, deferredList).done(function(){
                    targetBody.html(allRows);
                    // remove these handlers
                    $(document).off('click.wndoutside');

                    currentTableManager.WNDaddSelect2(targetBody.find('.WNDapplicableUsers'));
                    currentTableManager.attachDeleteHandlers();
                    currentTableManager.attachEditHandlers();
                    targetBody.find("select.wnd-select").select2({minimumResultsForSearch: -1, width: '160px'}).change();
                });
            }
        }, function(jqhrx, errorStatus, errorMessage){
                OC.dialogs.alert(escapeHTML(jqhrx.status + ": " + errorMessage),
                          escapeHTML(t('windows_network_drive', 'Internal error')));

        });
    };

    this.addToTable = function(mount) {
        var codeGen = this.codeGenerator;
        var targetBody = this.tbody;
        var currentTableManager = this;
        if (!mount.applicable_values) {
            mount.applicable_values = 'All users';
        }
        codeGen.generateTableRow2(mount, function(template){
            targetBody.append(template);
            currentTableManager.WNDaddSelect2(template.find('.WNDapplicableUsers'));
            currentTableManager.attachDeleteHandlers(template);
            currentTableManager.attachEditHandlers(template);
            currentTableManager._postAddMountFix(template, mount);
            template.find("select.wnd-select").select2({minimumResultsForSearch: -1, width: '160px'}).change();
        }, function(jqhrx, errorStatus, errorMessage){
                OC.dialogs.alert(escapeHTML(jqhrx.status + ": " + errorMessage),
                          escapeHTML(t('windows_network_drive', 'Internal error')));

        });
    }

    this.extraClearInputs = function(baseElement) {
        baseElement.find("input, select").each(function(){
            var thisElement = $(this);
            if (thisElement.is("select")) {
                thisElement.find('option').prop("selected", false)
                                    .filter('[value="global"],[selected]')
                                    .prop("selected", true);
                thisElement.trigger("liszt:updated").change();
            } else if (thisElement.attr("type") === "text" || thisElement.attr("type") === "password") {
                thisElement.val('').change();
            } else if (thisElement.attr("type") === "checkbox") {
                thisElement.prop("checked", false).change();
            }
        });
    };

    this.saveData = function(baseElement) {
        var allData = {};

        baseElement.find("input, select").each(function(){
            var thisElement = $(this);
            if (thisElement.is("select[multiple]")) {
                var values = thisElement.val() || [];
                allData[thisElement.attr("name")] = values.join(",");
            } else if (thisElement.attr("type") === "text" || thisElement.attr("type") === "password" || thisElement.is("select:not([multiple])")) {
                allData[thisElement.attr("name")] = thisElement.val();
            } else if (thisElement.attr("type") === "checkbox") {
                allData[thisElement.attr("name")] = thisElement.prop("checked");
            }
        });

        var currentTableManager = this;
        this.wndAjaxCalls.save(allData, function(response){
            if (response.status === "success") {
                currentTableManager.extraClearInputs(baseElement);
                if (response.id !== null) {
                    var mountData = {
                        mount_id: response.id,
                        mount_point: allData.WNDMountPoint,
                        applicable_values: allData.WNDMountType,
                        mount_url: allData.WNDUrl,
                        share: allData.WNDShare,
                        root: allData.WNDRoot,
                        credType: allData.WNDCredentialsType,
                        global_username: allData.WNDGuser
                    };
                    currentTableManager.addToTable(mountData);
                } else {
                    console.log('no id found, reloading all the table');
                    currentTableManager.reloadTable();
                }
            } else {
                if (response.data && response.data.message) {
                    OC.dialogs.alert(response.data.message,
                            escapeHTML(t('windows_network_drive', 'Error checking Windows network drive configuration')));
                } else {
                    OC.dialogs.alert(escapeHTML(t('windows_network_drive', 'Internal server error')),
                                escapeHTML(t('windows_network_drive', 'Error checking Windows network drive configuration')));
                }
            }
        }, function(jqhrx, errorStatus, errorMessage){
                OC.dialogs.alert(escapeHTML(jqhrx.status + ": " + errorMessage),
                          escapeHTML(t('windows_network_drive', 'Internal error')));

        });
    };


    this.WNDaddSelect2 = function($elements) {
        var userListLimit = 30;
        var cachedUserResults = {};
        if ($elements.length > 0) {
            $elements.select2({
                placeholder: t('windows_network_drive', 'All users. Type to select user or group.'),
                allowClear: true,
                multiple: true,
                //minimumInputLength: 1,
                ajax: {
                    url: OC.filePath('windows_network_drive', 'ajax', 'applicable.php'),
                    dataType: 'json',
                    quietMillis: 100,
                    data: function (term, page) { // page is the one-based page number tracked by Select2
                        return {
                            pattern: term, //search term
                            limit: userListLimit, // page size
                            offset: userListLimit*(page-1) // page number starts with 0
                        };
                    },
                    results: function (data, page) {
                        if (data.status === "success") {
                            var results = [];
                            var userCount = 0; // users is an object
                            // add groups
                            $.each(data.groups, function(i, group) {
                                results.push({name:group+'(group)', displayname:group, type:'group' });
                            });
                            // add users
                            $.each(data.users, function(id, user) {
                                userCount++;
                                results.push({name:id+'(user)', displayname:user, type:'user' });
                            });

                            results.push({name:'global', displayname:'All users', type:'global'});

                            var more = (userCount >= userListLimit) || (data.groups.length >= userListLimit);
                            return {results: results, more: more};
                        } else {
                            //FIXME add error handling
                        }
                    }
                },
                initSelection: function(element, callback) {
                    var promises = [];
                    var results = [];

                    $(element.val().split(",")).each(function (i,userId) {

                        var def = new $.Deferred();
                        promises.push(def.promise());
                        if (userId === 'All users') {
                            results.push({name:'global', displayname:'All users', type:'global'});
                            def.resolve();
                            return;
                        }

                        var userpos = userId.lastIndexOf('(user)');
                        var grouppos = userId.lastIndexOf('(group)');
                        var pos = Math.max(userpos, grouppos);
                        if (pos === grouppos && pos !== -1) {
                            // it's a group
                            results.push({name:userId, displayname:userId.substr(0, pos), type:'group'});
                            def.resolve();
                        } else {
                            if (cachedUserResults[userId]) {
                                results.push(cachedUserResults[userId]);
                                def.resolve();
                                return;
                            }

                            var goodUserId = userId.substr(0, pos);
                            $.ajax(OC.filePath('windows_network_drive', 'ajax', 'applicable.php'), {
                                cache: true,
                                data: {
                                    pattern: goodUserId
                                },
                                dataType: "json"
                            }).done(function(data) {
                                if (data.status === "success") {
                                    if (data.users[goodUserId]) {
                                        var userObject = {name:userId, displayname:data.users[goodUserId], type:'user'};
                                        results.push(userObject);
                                        cachedUserResults[userId] = userObject;
                                    }
                                } else {
                                    //FIXME add error handling
                                }
                            }).always(function(){
                                // resolve the deferred always
                                def.resolve();
                            });
                        }
                    });
                    $.when.apply(undefined, promises).then(function(){
                        callback(results);
                    });
                },
                id: function(element) {
                    return element.name;
                },
                formatResult: function (element) {
                    var $result = $('<span><div class="avatardiv"/><span>'+escapeHTML(element.displayname)+'</span></span>');
                    var $div = $result.find('.avatardiv')
                                        .attr('data-type', element.type)
                                        .attr('data-name', element.name)
                                        .attr('data-displayname', element.displayname);
                    if (element.type === 'group') {
                        var url = OC.imagePath('core','places/contacts-dark'); // TODO better group icon
                        $div.html('<img width="32" height="32" src="'+url+'">');
                    }
                    return $result.get(0).outerHTML;
                },
                formatSelection: function (element) {
                    if (element.type === 'group') {
                        return '<span title="'+escapeHTML(element.name)+'" class="group">'+escapeHTML(element.displayname+' '+t('windows_network_drive', '(group)'))+'</span>';
                    } else {
                        return '<span title="'+escapeHTML(element.name)+'" class="user">'+escapeHTML(element.displayname)+'</span>';
                    }
                },
                escapeMarkup: function (m) {
                    // we escape the markup in formatResult and formatSelection
                    return m;
                },
                dropdownCssClass: 'WND-Select2-Dropdown'
            }).on("select2-loaded", function() {
                $.each($(".avatardiv"), function(i, div) {
                    $div = $(div);
                    if ($div.data('type') === 'user') {
                        $div.avatar($div.data('name'),32);
                    }
                });
            });
        }
    };
}


$(document).ready(function(){
    $('select.wnd-select').select2({minimumResultsForSearch: -1, width: '160px'});

    var tableManager = new TableManager($('#WNDadminMountPoints table tbody'));

    $('#WNDSaveButton').click(function() {
        tableManager.saveData($(this).parent().parent());
        // base element should be the tfoot's tr
    });

    var wndTypingTimer = {WNDGlobalUsername : null, WNDGlobalPassword : null};
    $('#WNDGlobalUsername, #WNDGlobalPassword').on('keyup input', function(){
        var _self = $(this);
        var _self_id = _self.attr('id');
        var map = {WNDGlobalUsername : 'WNDuser', WNDGlobalPassword : 'WNDpass'};

        clearTimeout(wndTypingTimer[_self_id]);
        wndTypingTimer[_self_id] = setTimeout(function() {

            var dataToSend = {type: 'admin'};
            dataToSend[map[_self_id]] = _self.val();

            $.post(OC.filePath('windows_network_drive', 'ajax', 'saveGlobalCredentials.php'),
                dataToSend,
                function(response){
                    if (response.status === 'success') {
                        OCA.WND.WNDUtils.showAlert(t('windows_network_drive', 'Credentials stored'));
                    } else {
                        OC.dialogs.message(t('windows_network_drive', 'Something went wrong saving the credentials'), t('windows_network_drive', 'Failure'));
                    }
                });
        }, 750);
    }).on('click', function(){
        $(this).val('').trigger('input');
    });

    $('#WNDCredentialsType').change(function(){
        var thisElement = $(this);
        if (thisElement.val() === "0") {
            thisElement.siblings('#WNDGuser, #WNDGpass').each(function(){
                $(this).show();
            });
        } else {
            thisElement.siblings('#WNDGuser, #WNDGpass').each(function(){
                $(this).hide();
            });
        }
    });

    var activationData = {'WNDActivatePersonal': {key: 'personal_mounts',
                                                    message: t('windows_network_drive', 'Are you sure you want to disable personal mountpoints? By doing this, all the personal mountpoints will be removed for all the users')},
                        'WNDActivateSharing': {key: 'global_sharing',
                                                message: t('windows_network_drive', 'Are you sure you want to disable shared mountpoints? By doing this, all the shared mountpoints will be removed for all the users')}};

    $('#WNDActivatePersonal, #WNDActivateSharing').change(function(){
        var value = this.checked;
        var thisElement = $(this);
        var sendRequest = function(value) {
            $.post(OC.filePath('windows_network_drive', 'ajax', 'activateKey.php'),
                    {key: activationData[thisElement.attr('id')].key, activate: value},
                    function(response) {
                        if (typeof response.status !== "undefined" && response.status !== "success") {
                            OC.dialogs.alert(response.data.message,
                                    escapeHTML(t('windows_network_drive', 'Error activating personal mounts')));
                        }
                    });
            };

        if (!this.checked) {
            thisElement.prop('checked', true);
            OC.dialogs.info(activationData[thisElement.attr('id')].message, t('windows_network_drive', 'Please, confirm'), function(){
                sendRequest(value);
                thisElement.prop('checked', false);
            }, true);
        } else {
            sendRequest(value);
        }
    });

    tableManager.WNDaddSelect2($('#WNDadminMountPoints table .WNDapplicableUsers'));

    tableManager.attachDeleteHandlers();
    tableManager.attachEditHandlers();

    tableManager.wndAjaxCalls.preajaxCallback = function(){
        if (typeof this.showTimes === "undefined") {
            this.showTimes = 0;
        }
        if (this.showTimes > 0) {
            this.showTimes++;
            return;
        }

        var wndBaseDiv = $('#WNDadminMountPoints');
        if (wndBaseDiv.children('img.loading').length === 0) {
            wndBaseDiv.append('<img class="loading" style="display:none ; position:absolute" src="' + OC.imagePath('core', 'loading.gif') + '" />');
        }
        var loadingGif = $('#WNDadminMountPoints > img.loading');
        loadingGif.show().position({my: 'center center',
                            at: 'center center',
                            of: '#WNDadminMountPoints table.grid'});
        this.showTimes++;
    };
    tableManager.wndAjaxCalls.postajaxCallback = function(){
        if (this.showTimes > 1) {
            this.showTimes--;
            return;
        }
        $('#WNDadminMountPoints > img.loading').hide();
        this.showTimes--;
    };
});
