/**
 * ownCloud
 *
 * @author Juan Pablo Villafañez Ramos <jvillafanez@owncloud.com>
 * @author Jesus Macias Portela <jesus@owncloud.com>
 * @copyright (C) 2014 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

function WndAjaxCalls() {
    this.preajaxCallback = null;
    this.postajaxCallback = null;
    this.sendRequest = function(type, targetUrl, dataToSend, successCallback, errorCallback) {
        var _self = this;

        if (typeof this.preajaxCallback === 'function') {
            this.preajaxCallback();
        }

        $.ajax({
            type : type,
            url : targetUrl,
            data : dataToSend,
            success : function(response) {
                if (successCallback && typeof successCallback === 'function') {
                    successCallback(response);
                }
            },
            error : function(jqxhr, state, error) {
                if (errorCallback && typeof errorCallback === 'function') {
                    errorCallback(jqxhr, state, error);
                }
            },
            complete : function() {
                if (typeof _self.postajaxCallback === 'function') {
                    _self.postajaxCallback();
                }
            }
        });
    };

    this.save = function(dataToSend, successCallback, errorCallback) {
        this.sendRequest('POST',
                        OC.filePath('windows_network_drive', 'ajax', 'saveMountPoint.php'),
                        dataToSend,
                        successCallback,
                        errorCallback);
    };

    this.savePersonalMount = function(dataToSend, successCallback, errorCallback) {
        this.sendRequest('POST',
                        OC.filePath('windows_network_drive', 'ajax', 'savePersonalMountPoint.php'),
                        dataToSend,
                        successCallback,
                        errorCallback);
    };

    this.updatePersonalMount = function(id, user, passwd, successCallback, errorCallback) {
        this.sendRequest('POST',
                        OC.filePath('windows_network_drive', 'ajax', 'updatePersonalMountPoint.php'),
                        {WNDMountId: id, WNDuser: user, WNDpass: passwd},
                        successCallback,
                        errorCallback);
    };

    this.updatePersonalMountWithType = function(id, type, user, passwd, successCallback, errorCallback) {
        this.sendRequest('POST',
                        OC.filePath('windows_network_drive', 'ajax', 'updatePersonalMountPoint.php'),
                        {WNDMountId: id, type: type, WNDuser: user, WNDpass: passwd},
                        successCallback,
                        errorCallback);
    };

    this.getAllMounts = function(successCallback, errorCallback) {
        this.sendRequest('GET',
                        OC.filePath('windows_network_drive', 'ajax', 'getMountPoint.php'),
                        {WNDMountType: 'global'},
                        successCallback,
                        errorCallback);
    };

    this.getMountsForUser = function(type, successCallback, errorCallback) {
        this.sendRequest('GET',
                        OC.filePath('windows_network_drive', 'ajax', 'getMountPointForUser.php'),
                        {type: type},
                        successCallback,
                        errorCallback);
    };

    this.deleteMount = function(id, successCallback, errorCallback) {
        this.sendRequest('POST',
                        OC.filePath('windows_network_drive', 'ajax', 'deleteMountPoint.php'),
                        {WNDMountId: id},
                        successCallback,
                        errorCallback);
    };

    this.deletePersonalMount = function(id, successCallback, errorCallback) {
        this.sendRequest('POST',
                        OC.filePath('windows_network_drive', 'ajax', 'deletePersonalMountPoint.php'),
                        {WNDMountId: id},
                        successCallback,
                        errorCallback);
    };

    this.updateMount = function(id, user, passwd, successCallback, errorCallback) {
        this.sendRequest('POST',
                        OC.filePath('windows_network_drive', 'ajax', 'updateMountPoint.php'),
                        {WNDMountId: id, WNDGuser: user, WNDGpass: passwd},
                        successCallback,
                        errorCallback);
    };

    this.updateMountWithType = function(id, type, user, passwd, successCallback, errorCallback) {
        this.sendRequest('POST',
                        OC.filePath('windows_network_drive', 'ajax', 'updateMountPoint.php'),
                        {WNDMountId: id, type: type, WNDGuser: user, WNDGpass: passwd},
                        successCallback,
                        errorCallback);
    };

    this.saveUserCredentials = function(id, user, passwd, successCallback, errorCallback) {
        this.sendRequest('POST',
                        OC.filePath('windows_network_drive', 'ajax', 'saveCredential.php'),
                        {WNDMountId: id, WNDuser: user, WNDpass: passwd},
                        successCallback,
                        errorCallback);
    };
}
