<?php
/**
 * ownCloud
 *
 * @author Juan Pablo Villafañez Ramos <jvillafanez@owncloud.com>
 * @author Jesus Macias Portela <jesus@owncloud.com>
 * @copyright (C) 2014 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

namespace OCA\windows_network_drive\lib;

use \OCA\windows_network_drive\lib\DBAccess;
use \OCA\windows_network_drive\lib\WNDUtils;

/**
 * Class to manage the required hooks for the app
 */
class Hooks {
    /**
     * Hook to mount the WND FS
     *
     * @param array $params
     */
    public static function mount_storages($params) {
        $user = $params['user'];
        $groups = \OC_Group::getUserGroups($user);
        $mountList = DBAccess::getMountsForUser($user, $groups);
        if ($mountList !== false) {
            while (($row = $mountList->fetchRow()) !== false) {
                WNDUtils::mountGlobalWndRow($user, $row);
            }
        }

        $personalMountList = DBAccess::getPersonalMountsPerUser($user);
        if ($personalMountList !== false) {
            while (($row = $personalMountList->fetchRow()) !== false) {
                WNDUtils::mountPersonalWndRow($user, $row);
            }
        }

        \OCP\Util::addScript('windows_network_drive', 'wndutils');
        \OCP\Util::addScript('windows_network_drive', 'mountManager');
        \OCP\Util::addScript('windows_network_drive', 'wndfs');
        \OCP\Util::addScript('windows_network_drive', 'rollingQueue');
    }

    /**
     * Intercepts the user credentials on login and stores them
     * encrypted inside the session if WND storage is enabled.
     * @param array $params
     */
    public static function login($params) {
        $hasWNDMount = false;
        $userWNDMounts = DBAccess::getMountsForUser($params['uid'], array());
        if ($userWNDMounts !== false) {
            while (($row = $userWNDMounts->fetchRow()) !== false) {
                if ($row['use_custom_credentials'] == 3) {
                    $hasWNDMount = true;
                    break;
                }
            }
        }

        if ($hasWNDMount) {
            \OC::$server->getSession()->set('wnd-credentials', \OC::$server->getCrypto()->encrypt(json_encode($params)));
        }
    }
}
