(function() {  
  var Page = require('../helper/page.js');
  var AdminPage = require('../pages/admin.page.js');

  var adminPage;
  
  var WindowsDrivePage = function(baseUrl) {
    adminPage = new AdminPage(baseUrl);    
    
    this.baseUrl = baseUrl;
    this.path = adminPage.path;
    this.url = baseUrl + this.path;
    
    //================ ELEMENTS ============================================================//

    this.wndSettings = element(by.id('WNDadmin'));
    this.wndInputs = this.wndSettings.element(by.css('tfoot'));
    this.globalUserNameInput = this.wndSettings.element(by.id('WNDGlobalUsername'));
    this.globalPasswordInput = this.wndSettings.element(by.id('WNDGlobalPassword'));
    this.mountPointInput = this.wndInputs.element(by.id('WNDMountPoint'));
    this.availableForInput = this.wndInputs.element(by.css('#s2id_WNDMountType input'));
    this.urlInput = this.wndInputs.element(by.id('WNDUrl'));
    this.shareInput = this.wndInputs.element(by.id('WNDShare'));
    this.rootInput = this.wndInputs.element(by.id('WNDRoot'));
    this.credentialsTypeDropdown = this.wndInputs.element(by.id('WNDCredentialsType_chzn'));
    this.credentialsTypeSelect = [];
    this.credentialsTypeSelect[0] = this.wndInputs.element(by.id('WNDCredentialsType_chzn_o_0'));
    this.credentialsTypeSelect[1] = this.wndInputs.element(by.id('WNDCredentialsType_chzn_o_1'));
    this.credentialsTypeSelect[2] = this.wndInputs.element(by.id('WNDCredentialsType_chzn_o_2'));
    this.gUserInput = this.wndInputs.element(by.id('WNDGuser'));
    this.gPasswordInput = this.wndInputs.element(by.id('WNDGpass'));
    this.personalCheckbox = this.wndSettings.element(by.id('WNDActivatePersonal'));
    this.sharingCheckbox = this.wndSettings.element(by.id('WNDActivateSharing'));

    this.saveButton = this.wndSettings.element(by.id('WNDSaveButton'));
    // this.editButton = element(by.name('edit'));
    this.deleteButton = element(by.name('delete'));
    
    this.ocDialogTitle = function() {
      return element(by.css('h3.oc-dialog-title'));
    };
    
    this.mountPoints = this.wndSettings.all(by.id('WNDadminMountPoints'));
  };


  //================ NAVAGATION ==========================================================//

  WindowsDrivePage.prototype.getAsUser = function(userName, pass) {
    return adminPage.getAsUser(userName, pass);
  };

  WindowsDrivePage.prototype.get = function() {
    return adminPage.get();
  };

  // Save function assumes that no mount is configured before
  WindowsDrivePage.prototype.save = function() {
    var mounts = this.mountPoints;
    var nrMounts = 0;
    
    this.saveButton.click();
    
    var dialog = this.ocDialogTitle();
    return browser.wait(function() {
      var deferred = protractor.promise.defer();

      dialog.isPresent().then(function (isPresent) {
        mounts.count().then(function(i) {
          deferred.fulfill( (i == 1) || isPresent);
        });
      });
      return deferred.promise;
    }, browser.params.wait);
  };
  
  WindowsDrivePage.prototype.globalUserName = function(user) {
    this.globalUserNameInput.clear();
    this.globalUserNameInput.sendKeys(user);
  };
  
  WindowsDrivePage.prototype.globalPassword = function(user) {
    this.globalPasswordInput.clear();
    this.globalPasswordInput.sendKeys(user);
  };
  
  //================ ACTIVATE / DISABLE OPTIONS ==========================================//

  // activates personal option
  WindowsDrivePage.prototype.activatePersonal = function() {
    return adminPage.activateOption(this.personalCheckbox);
  };

  // activates sharing option
  WindowsDrivePage.prototype.activateSharing = function() {
    return adminPage.activateOption(this.sharingCheckbox);
  };  

  // disables personal option
  WindowsDrivePage.prototype.disablePersonal = function() {
    return adminPage.disableOption(this.personalCheckbox);
  };

  // disables sharing option
  WindowsDrivePage.prototype.disableSharing = function() {
    return adminPage.disableOption(this.sharingCheckbox);
  };
  
  WindowsDrivePage.prototype.selectCredentials = function(option_index) {
    this.credentialsTypeDropdown.click();
    this.credentialsTypeSelect[option_index].click();
  };

  module.exports = WindowsDrivePage;
})();