<?php
/**
 * ownCloud
 *
 * @author Juan Pablo Villafañez Ramos <jvillafanez@owncloud.com>
 * @author Jesus Macias Portela <jesus@owncloud.com>
 * @copyright (C) 2014 ownCloud, Inc.
 *
 * This code is covered by the ownCloud Commercial License.
 *
 * You should have received a copy of the ownCloud Commercial License
 * along with this program. If not, see <https://owncloud.com/licenses/owncloud-commercial/>.
 *
 */

namespace OCA\windows_network_drive\tests;

use \OCA\windows_network_drive\lib\Utils;

class utilsTest extends \PHPUnit_Framework_TestCase {
    public function test_passwordCheck() {
        // password is encrypted using some random bytem so we cannot ensure that a string
        // will be encrypted to another one. Plus we cannot ensure the same string will
        // be encrypted the same way (and return the same result)
        $testString = 'qwerty1234=)(/&%';
        $this->assertEquals($testString, Utils::decryptPassword(Utils::encryptPassword($testString)));
        $this->assertFalse(Utils::decryptPassword('badpassword'));
    }

    public function test_getDefault() {
        $testArray = array('key1' => 'value1', 'key2' => null);
        $this->assertEquals('value1', Utils::getDefault($testArray, 'key1'));
        $this->assertEquals('value1', Utils::getDefault($testArray, 'key1', false));
        $this->assertNull(Utils::getDefault($testArray, 'key2'));
        $this->assertNull(Utils::getDefault($testArray, 'missing'));
        $this->assertFalse(Utils::getDefault($testArray, 'key2', false));
        $this->assertFalse(Utils::getDefault($testArray, 'missing', false));
    }

    public function test_invalidChars() {
        $value = false;
        Utils::checkInvalidChars('string_bad', '_-.:', function($e) use (&$value) {
            $value = true;
        });
        $this->assertTrue($value);

        $value2 = false;
        Utils::checkInvalidChars('stringgood', '_-.:', function($e) use (&$value2) {
            $value2 = true;
        });
        $this->assertFalse($value2);
    }
}
